function [nwtn, rlrs, pcgi]  = tv_pcg_wrapper(PCG, Hessian, g, opts)

% Define preconditioner function.
if opts.mu <= opts.prec_tol;
        
    scale_1_mat = sparse(1:opts.n,1:opts.n,PCG.scale_1);
    scale_2_mat = sparse(1:opts.n,1:opts.n,PCG.scale_2);
    scale_3_mat = sparse(1:opts.n,1:opts.n,PCG.scale_3);
    scale_4_mat = sparse(1:opts.n,1:opts.n,PCG.scale_4);
    P = opts.Wt_re'*(scale_1_mat*opts.Wt_re) + ...
        opts.Wt_im'*(scale_4_mat*opts.Wt_im) + ...
        0.5.*(opts.Wt_re'*(scale_2_mat*opts.Wt_im) + ...
        opts.Wt_im'*(scale_2_mat*opts.Wt_re)) + ...
        0.5.*(opts.Wt_im'*(scale_3_mat*opts.Wt_re) + ...
        opts.Wt_re'*(scale_3_mat*opts.Wt_im)) + ...
        opts.reg*PCG.I + PCG.rho*PCG.I;
            
    % If problem is quite large, then it is better to use the
    % preconditioner in an iterative way.
    if opts.n < 2^23
        Preconditioner = @(q) P\q; 
    else
        Preconditioner = @(q) prec_W(opts,q,H,PCG.rho,[],opts.prec_itrs);
    end                       
else 
    Preconditioner = [];   
end

[nwtn, rlrs, pcgi] = linear_solver(Hessian,-g,opts.tolpcg,...
                            opts.mxitrpcg,Preconditioner);
        
assignin('caller','PCG',PCG);
end